package net.wanji.web.service.scheme.impl;

import net.wanji.web.common.exception.WeekException;
import net.wanji.web.dto.RunningPlanDTO;
import net.wanji.web.mapper.scheme.CrossPlanMapper;
import net.wanji.web.mapper.scheme.CrossSchedulesMapper;
import net.wanji.web.mapper.scheme.CrossSchedulesPlanMapper;
import net.wanji.web.mapper.scheme.CrossSchemeMapper;
import net.wanji.web.mapper.scheme.CrossSectionMapper;
import net.wanji.web.po.scheme.CrossPlanPO;
import net.wanji.web.po.scheme.CrossSchedulesPO;
import net.wanji.web.po.scheme.CrossSchedulesPlanPO;
import net.wanji.web.po.scheme.CrossSchemePO;
import net.wanji.web.po.scheme.CrossSectionPO;
import net.wanji.web.service.scheme.RunningPlanService;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.Date;
import java.util.List;

/**
 * @author Kent HAN
 * @date 2023/1/28 16:41
 */
@Service
public class RunningPlanServiceImpl implements RunningPlanService {
    private final CrossPlanMapper crossPlanMapper;
    private final CrossSchemeMapper crossSchemeMapper;
    private final CrossSectionMapper crossSectionMapper;
    private final CrossSchedulesMapper crossSchedulesMapper;
    private final CrossSchedulesPlanMapper crossSchedulesPlanMapper;

    public RunningPlanServiceImpl(CrossPlanMapper crossPlanMapper, CrossSchemeMapper crossSchemeMapper,
                                  CrossSectionMapper crossSectionMapper, CrossSchedulesMapper crossSchedulesMapper,
                                  CrossSchedulesPlanMapper crossSchedulesPlanMapper) {
        this.crossPlanMapper = crossPlanMapper;
        this.crossSchemeMapper = crossSchemeMapper;
        this.crossSectionMapper = crossSectionMapper;
        this.crossSchedulesMapper = crossSchedulesMapper;
        this.crossSchedulesPlanMapper = crossSchedulesPlanMapper;
    }

    @Override
    @Transactional
    public void saveRunningPlan(RunningPlanDTO runningPlanDTO) {
        String crossId = runningPlanDTO.getCrossId();
        // 验证特殊日期
        checkSpecialDate(runningPlanDTO);
        // 清空之前计划
        crossPlanMapper.deleteByCrossId(crossId);
        crossSectionMapper.deleteByCrossId(crossId);
        crossSchedulesMapper.deleteByCrossId(crossId);
        crossSchedulesPlanMapper.deleteByCrossId(crossId);
        // 保存计划数据
        saveCrossPlan(runningPlanDTO, crossId);
        // 保存调度数据
        saveCrossSchedules(runningPlanDTO, crossId);
    }

    private void saveCrossSchedules(RunningPlanDTO runningPlanDTO, String crossId) {
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = runningPlanDTO.getSchedulesPlanList();
        for (RunningPlanDTO.SchedulesPlanListElement schedulesPlan : schedulesPlanList) {
            // 更新调度表获取调度ID
            CrossSchedulesPO crossSchedulesPO = new CrossSchedulesPO();
            crossSchedulesPO.setScheduleNo(schedulesPlan.getScheduleNo());
            crossSchedulesPO.setName(schedulesPlan.getName());
            crossSchedulesPO.setStatus(0); // 新增调度状态为未执行
            crossSchedulesPO.setCrossId(crossId);
            crossSchedulesMapper.insertOne(crossSchedulesPO);
            Integer crossSchedulesId = crossSchedulesPO.getId();
            // 根据路口ID和计划名获取计划ID
            String planName = schedulesPlan.getPlanName();
            CrossPlanPO crossPlanPO = crossPlanMapper.selectByCrossIdAndName(crossId, planName);
            Integer planId = crossPlanPO.getId();
            // 更新调度计划关系表
            updateCrossSchedulesPlan(crossId, schedulesPlan, crossSchedulesId, planId);
        }
    }

    private void updateCrossSchedulesPlan(String crossId, RunningPlanDTO.SchedulesPlanListElement schedulesPlan,
                                                 Integer crossSchedulesId, Integer planId) {
        List<Integer> week = schedulesPlan.getWeek();
        CrossSchedulesPlanPO crossSchedulesPlanPO = new CrossSchedulesPlanPO();
        crossSchedulesPlanPO.setCrossId(crossId);
        crossSchedulesPlanPO.setSchedulesId(crossSchedulesId);
        crossSchedulesPlanPO.setPlanId(planId);
        if (week == null || week.size() == 0) {
            crossSchedulesPlanPO.setSpecialDate(schedulesPlan.getSpecialDate());
        } else {
            for (Integer day : week) {
                crossSchedulesPlanPO.setWeek(day);
                crossSchedulesPlanPO.setSpecialDate(new Date(0));
            }
        }
        crossSchedulesPlanMapper.insertOne(crossSchedulesPlanPO);
    }

    private void saveCrossPlan(RunningPlanDTO runningPlanDTO, String crossId) {
        List<RunningPlanDTO.DailyPlanListElement> dailyPlanList = runningPlanDTO.getDailyPlanList();
        for (RunningPlanDTO.DailyPlanListElement dailyPlan : dailyPlanList) {
            // 更新计划表获取计划ID
            CrossPlanPO crossPlanPO = new CrossPlanPO();
            crossPlanPO.setPlanNo(dailyPlan.getPlanNo());
            crossPlanPO.setName(dailyPlan.getName());
            crossPlanPO.setCrossId(crossId);
            crossPlanMapper.insertOne(crossPlanPO);
            Integer crossPlanId = crossPlanPO.getId();
            // 根据路口ID和方案名获取方案ID
            String schemeName = dailyPlan.getSchemeName();
            CrossSchemePO crossSchemePO = crossSchemeMapper.selectByCrossIdAndSchemeName(crossId, schemeName);
            Integer schemeId = crossSchemePO.getId();
            // 更新时段表
            updateCrossSection(crossId, dailyPlan, crossPlanId, schemeId);
        }
    }

    private void updateCrossSection(String crossId, RunningPlanDTO.DailyPlanListElement dailyPlan, 
                                    Integer crossPlanId, Integer schemeId) {
        CrossSectionPO crossSectionPO = new CrossSectionPO();
        String startTime = dailyPlan.getStartTime();
        String endTime = dailyPlan.getEndTime();
        String s = startTime + endTime;
        String sectionNo = s.replaceAll(":", "");
        crossSectionPO.setSectionNo(sectionNo);
        crossSectionPO.setStartTime(startTime);
        crossSectionPO.setEndTime(endTime);
        crossSectionPO.setCrossId(crossId);
        crossSectionPO.setPlanId(crossPlanId);
        crossSectionPO.setControlMode(1); // todo 暂时只有定周期
        crossSectionPO.setSchemeId(schemeId);
        crossSectionMapper.insertOne(crossSectionPO);
    }

    private static void checkSpecialDate(RunningPlanDTO runningPlanDTO) {
        List<RunningPlanDTO.SchedulesPlanListElement> schedulesPlanList = runningPlanDTO.getSchedulesPlanList();
        for (RunningPlanDTO.SchedulesPlanListElement schedulesPlan : schedulesPlanList) {
            List<Integer> week = schedulesPlan.getWeek();
            Date specialDate = schedulesPlan.getSpecialDate();
            if (week != null && week.size() > 0 && specialDate != null) {
                throw new WeekException("不能同时有星期选择和日期选择");
            }
        }
    }
}
